
use Handle;
my %EXTANT;
sub extant () { scalar keys %EXTANT }

use Test::More tests => 9;
ok(1, "Partial credit for showing up");

my $O;
my $handle = Handle->new($O = TTree->new('Dummy'));
$EXTANT{$O} = 1; undef $O;
undef $handle;
is(extant, 0, "all objects destroyed");

# A tree with no handle should leak
$O = Tree->new('Parent', Tree->new('Child'));
note_all($O);
is(extant, 2, "created small tree of animals");
undef $O;
is(extant, 2, "leaked");
%EXTANT = ();

# Just breaking the links on the tree class 
# should NOT destroy the tree, because it breaks only the back links,
# not the forward links
$O = TTree->new('Parent', TTree->new('Child'));
note_all($O);
is(extant, 2, "created another small tree of animals");
$O->break_links;
is(extant, 2, "still alive after breaking links");
undef $O;
is(extant, 0, "breaking works");

# Here we create a big tree via a handle and then 
# dispose of the whole thing.  This tests the handle itself
$handle = Handle->new($O = TTree->new('Animal', 
                                      TTree->new('Dogs',
                                                 TTree->new('Snoopy'),
                                                 TTree->new('Lassie'),
                                                 TTree->new('Laika'),
                                                ),
                                      TTree->new('Cats',
                                                 TTree->new('Socks'),
                                                )
                                     )
                     );
note_all($O); undef $O;
is(extant, 7, "large tree");
# This destroys the handle, which should take care of disposing of
# the rest of the tree.
undef $handle;  
is(extant, 0, "large tree cleaned up");

sub note_all {
  my $self = shift;
  $EXTANT{"$self"} = 1;
  for my $node ($self->children) {
    note_all($node);
  }
}

package TTree;
use base Tree;

sub DESTROY {
  my $self = shift;
#  warn "Destroying $self ($self->{DATA})\n";
  if (delete $EXTANT{"$self"}) {
    return;
  } else {
    my $data = $self->data;
    warn "Non-extant object '$data' ($self) destroyed]\n";
  }
}
